# 01-Brightness-temperature.py.     I. Marti-Vidal.      IMPRS Blackboard Lectures 2011.
# MONTE CARLO ESTIMATE OF THE UNCERTAINTY IN THE OBSERVED BRIGHTNESS TEMPERATURE.

import numpy as np
import pylab as pl

# Some constants:

Jy = 1.e-23 # cgs
Mpc = 3.26*365.25*(3.e+10)*(1.e+6) # cm
Gig = 1.e-9

# Parameters:

Lambda = 3.6       # Observing wavelength (cm)
flux = 0.100       # Observed flux density (Jy)
errflux = 0.01     # Flux-density uncertainty (Jy)
dist = 3.63*Mpc    # Distance to the source (it is in M81).
errdist = 0.36*Mpc # Distance uncertainty.
size = 0.8         # Size of the source (mas).
errsize = 0.1      # Size uncertainty (mas).
ndata = 10000      # Number of Monte-Carlo simulations.

# True values and linear error propagation:

Tb =  (1.360e+9) *flux * np.power(Lambda/size,2.)
lum = (4.*np.pi*Jy) *flux * np.power(dist,2.)

errTb  = Tb *np.sqrt((errflux/flux)**2.+(2.*errsize/size)**2.)
errlum = lum*np.sqrt((errflux/flux)**2.+(2.*errdist/dist)**2.)


print "\n Number of points: ",ndata
print "\n - True brightness temp. (1.e+9 K): ",Tb*Gig," +/- ",errTb*Gig
print " - True luminosity (cgs): ",lum," +/- ",errlum

# Simulations:

flvec = np.random.normal(flux,errflux,ndata)
sizevec =  np.random.normal(size,errsize,ndata)
distvec = np.random.normal(dist,errdist,ndata)

Tbvec = (1.360e+9) * flvec* np.power(Lambda/sizevec,2.)
lumvec = (4.*np.pi*Jy)* flvec* np.power(distvec,2.)

print "\n - MC brightness temp. (1.e+9 K): ",np.average(Tbvec)*Gig," +/- ",np.std(Tbvec)*Gig
print " - MC luminosity (cgs): ",np.average(lumvec)," +/- ",np.std(lumvec),"\n"


